#pragma once
#include <Eigen/Core>
#include <string>
#include <igl/readSTL.h>
#include <igl/slice.h>
#include <igl/slice_into.h>
#include "stiffness_matrix_assembly.hpp"
#include "load_vector_assembly.hpp"
#include "dirichlet_boundary.hpp"

typedef Eigen::VectorXd Vector;


//! Solve the FEM system.
//!
//! @param[out] u will at the end contain the FEM solution.
//! @param[in] vertices list of triangle vertices for the mesh
//! @param[in] triangles list of triangles (described by indices)
//! @param[in] f the RHS f (as in the exercise)
//! @param[in] g the boundary value (as in the exercise)
//! return number of degrees of freedom (without the boundary dofs)
int solveFiniteElement(Vector& u,
    const Eigen::MatrixXd& vertices,
    const Eigen::MatrixXi& triangles,
    const std::function<double(double, double)>& f,
    const std::function<double(double, double)>& g) 
{
    // Compute stiffnessmatrix A and load vector F

    u.resize(vertices.rows());
    u.setZero();

    // Compute dirichlet boundary conditions and apply it to F

    // To make a matrix from A that only consists of the interior points,
    // you can do
    //SparseMatrix AInterior;
    //igl::slice(A, interiorVertexIndices, interiorVertexIndices, AInterior);

    // And the same for a vector:
    // Vector FInterior;
    // igl::slice(F, interiorVertexIndices, FInterior);

    
    // To get the vector from the interior nodes to all nodes, you can use:
    // igl::slice_into(uInterior, interiorVertexIndices, u);
    

    // CHANGE LINE BELOW!!!
    return 0; // Return proper degrees of freedom here!
    // CHANGE LINE ABOVE!!!
}
