#pragma once
#include <Eigen/Core>
#include "load_vector.hpp"

//! Assemble the load vector into the full right hand side
//! for the linear system
//!
//! @param[out] F will at the end contain the RHS values for each vertex.
//! @param[in] vertices a list of triangle vertices
//! @param[in] triangles a list of triangles
//! @param[in] f the RHS function f.
void assembleLoadVector(Eigen::VectorXd& F,
                        const Eigen::MatrixXd& vertices,
                        const Eigen::MatrixXi& triangles,
                        const std::function<double(double, double)>& f)
{
    F.resize(vertices.rows());
    F.setZero();
    
    // Assemble the stiffness matrix here.
    // NOTE: For each triangle i, you can get the
    // three triangle corners (vertices) with:
    //
    // const auto& indexSet = triangles.row(i)
    // const auto& a = vertices.row(indexSet(0))
    // const auto& b = vertices.row(indexSet(1))
    // const auto& c = vertices.row(indexSet(2))
}
