#pragma once
#include <iostream>
#include <Eigen/Core>
#include <igl/readMESH.h>
#include <igl/readDMAT.h>
#include "writer.hpp"
#include "fem_solve.hpp"

double f_square(double x, double y) {
    return 2 * M_PI * M_PI * sin(M_PI *x) * sin(M_PI * y);
}

double g_square(double x, double y) {
    return 0;
}

double u_ex(double x, double y) {
    return sin(M_PI *x) * sin(M_PI * y);
}

Eigen::Vector2d u_ex_grad(double x, double y) {
  Eigen::Vector2d gradient;
  gradient << M_PI*cos(M_PI*x)*sin(M_PI*y), M_PI*sin(M_PI*x)*cos(M_PI*y);
  return gradient;
}

void solveSquare() {
    std::cout << "Solving square" << std::endl;
    Vector u;

    Eigen::MatrixXd vertices;
    Eigen::MatrixXi triangles;
    Eigen::MatrixXi tetrahedra;

    igl::readMESH(CMEA2_DATA_PATH "square_5.mesh", vertices, tetrahedra, triangles);

    solveFiniteElement(u, vertices, triangles, f_square, g_square);

    writeToFile("square_5_values.txt", u);
    writeMatrixToFile("square_5_vertices.txt", vertices);
    writeMatrixToFile("square_5_triangles.txt", triangles);
}
